@@ -1,208 +1,220 @@
-package Tk::MesgBox ;    # Documented at the __END__.
+package Tk::MesgBox;    # Documented at the __END__.
 
-# $Id: MesgBox.pm,v 1.10 2000/05/05 16:42:37 root Exp $
+# $Id: MesgBox.pm,v 1.12 2000/09/16 14:30:13 mark Exp mark $
 
-require 5.004 ;
+require 5.004;
 
-use strict ;
+use strict;
 
-use Tk ;
-use Carp ;
+use Tk;
+use Carp;
 
-require Tk::Toplevel ;
+require Tk::Toplevel;
 
-use vars qw( $VERSION @ISA ) ;
+use vars qw($VERSION @ISA);
 
-$VERSION = '1.11' ;
+$VERSION = '1.12';
 
-@ISA = qw( Tk::Toplevel ) ;
+@ISA = qw(Tk::Toplevel);
 
-Construct Tk::Widget 'MesgBox' ;
+Construct Tk::Widget 'MesgBox';
 
 #############################
 sub Populate {
-    my( $win, $args ) = @_ ;
+    my ($win, $args) = @_;
 
     my %arg = (
-        -title     => 'Message',
-        -text      => 'Press OK to continue...',
-        -icon      => undef,
+        -title => 'Message',
+        -text => 'Press OK to continue...',
+        -textfg => 'black',
+        -textbg => 'lightgrey',
+        -icon => undef,
         -defbutton => 'OK',    
-        -canbutton => '', # Usually 'Cancel'.
-        -buttons   => [ 'OK' ],
-        -aspect    => 500,
-        -justify   => 'center',
-        -bindctrl  => 1,
-        -bindalt   => 1,
-        -bindord   => 1,
+        -canbutton  => '', # Usually 'Cancel'.
+        -buttons => ['OK'],
+        -buttonpadx => 0,
+        -buttonpady => 0,
+        -buttonfg => 'black',
+        -buttonbg => 'lightgrey',
+        -buttonHL => 'lightgrey',
+        -buttonfont => '-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*',
+        -aspect => 500,
+        -justify => 'center',
+        -bindctrl => 1,
+        -bindalt => 1,
+        -bindord => 1,
         %$args, 
-        ) ;
+        );
 
     # We don't delete the title.
-    delete $args->{-text} ;
-    delete $args->{-icon} ;
-    delete $args->{-defbutton} ;
-    delete $args->{-canbutton} ;
-    delete $args->{-buttons} ;
-    delete $args->{-aspect} ;
-    delete $args->{-justify} ;
-    delete $args->{-bindctrl} ;
-    delete $args->{-bindalt} ;
-    delete $args->{-bindord} ;
+    delete @{$args}{qw(
+        -text -textfg -textbg -icon -defbutton -canbutton
+        -buttons -aspect -justify -bindctrl -bindalt
+        -bindord -buttonpadx -buttonpady -buttonfg
+        -buttonbg -buttonfont -buttonHL -buttonfont
+        )};
 
-    $win->SUPER::Populate( $args ) ;
+    $win->SUPER::Populate($args);
 
-    $win->withdraw ;
-    $win->iconname( $arg{-title} ) ;
-    $win->protocol( 'WM_DELETE_WINDOW' => sub { } ) ;
-    $win->transient( $win->toplevel ) ;
+    $win->withdraw;
+    $win->iconname($arg{-title});
+    $win->protocol('WM_DELETE_WINDOW' => sub { });
+    $win->transient($win->toplevel);
 
-    $win->{-default_button}  = &create_buttons( $win, %arg ) ;
-    &create_text_frame( $win, %arg ) ;
+    $win->{-default_button} = &create_buttons($win, %arg);
+    &create_text_frame($win, %arg);
 
-    $win->{-selected_button} = '' ;
+    $win->{-selected_button} = '';
 }
 
 #############################
 sub Show {
-    my $win = shift ;
+    my $win = shift;
 
-    $win->Popup() ; 
+    $win->Popup(); 
 
-    if( defined $win->{-default_button} ) {
-        $win->{-default_button}->focus ;
+    if (defined $win->{-default_button}) {
+        $win->{-default_button}->focus;
     } 
     else {
-      $win->focus ;
+      $win->focus;
     }
 
-    $win->waitVariable( \$win->{-selected_button} ) ;
-    $win->withdraw ;
+    $win->waitVariable(\$win->{-selected_button});
+    $win->withdraw;
 
-    $win->{-selected_button} ;
+    $win->{-selected_button};
 }
 
 #############################
 sub create_buttons {
-    my $win = shift ;
-    my %arg = @_ ;
+    my $win = shift;
+    my %arg = @_;
 
-    my $defbutton = undef ;
+    my $defbutton = undef;
 
     my $ButtonFrame = $win->Frame(
-                        )->pack( -padx => 5, -pady => 5, -side => 'bottom' ) ;
+                        )->pack(-padx => 5, -pady => 5, -side => 'bottom');
 
-    foreach my $button ( @{$arg{-buttons}} ) {
-        next unless $button ; 
+    foreach my $button (@{$arg{-buttons}}) {
+        next unless $button; 
         my $Button = $ButtonFrame->Button(
-            -text      => $button,
+            -text => $button,
             -underline => 0,
-            -width     => 8,
-            -command   => [ \&msg_close, $win, $button ],
-            )->pack( -side => 'left' ) ;
-        $defbutton = $Button if $button eq $arg{-defbutton} ;
+            -width => 8,
+            -command => [\&msg_close, $win, $button],
+            -foreground => $arg{-buttonfg},
+            -background => $arg{-buttonbg},
+            -padx => $arg{-buttonpadx},
+            -pady => $arg{-buttonpady},
+            -font => $arg{-buttonfont},
+            -activebackground => $arg{-buttonHL},
+            )->pack(-side => 'left');
+        $defbutton = $Button if $button eq $arg{-defbutton};
      }
 
-    &set_key_bindings( $win, %arg ) ;
+    &set_key_bindings($win, %arg);
 
-    $defbutton ;
+    $defbutton;
 }
 
 #############################
 sub set_key_bindings {
-    my $win = shift ;
-    my %arg = @_ ;
+    my $win = shift;
+    my %arg = @_;
 
-    foreach my $button ( @{$arg{-buttons}} ) {
-        next unless $button ; 
-        my $char = lc substr( $button, 0, 1 ) ;
-        $win->bind( "<Control-${char}>", [ \&msg_close, $win, $button ] ) 
-        if $arg{-bindctrl} ;
-        $win->bind( "<Alt-${char}>",     [ \&msg_close, $win, $button ] )
-        if $arg{-bindalt} ;
-        $win->bind( "<${char}>",         [ \&msg_close, $win, $button ] )
-        if $arg{-bindord} ;
-        $win->bind( "<Return>",          [ \&msg_close, $win, $button ] ) 
-        if $button eq 'OK' ;
-        $win->bind( "<Escape>",          [ \&msg_close, $win, $button ] ) 
-        if $button eq 'Cancel' ;
+    foreach my $button (@{$arg{-buttons}}) {
+        next unless $button; 
+        my $char = lc substr($button, 0, 1);
+        $win->bind("<Control-${char}>", [\&msg_close, $win, $button]) 
+        if $arg{-bindctrl};
+        $win->bind("<Alt-${char}>", [\&msg_close, $win, $button])
+        if $arg{-bindalt};
+        $win->bind("<${char}>", [\&msg_close, $win, $button])
+        if $arg{-bindord};
+        $win->bind("<Return>", [\&msg_close, $win, $button]) 
+        if $button eq 'OK';
+        $win->bind("<Escape>", [\&msg_close, $win, $button]) 
+        if $button eq 'Cancel';
     }
 
-    $win->bind( '<Return>', [ \&msg_close, $win, $arg{-defbutton} ] ) 
-    if $arg{-defbutton} ; 
+    $win->bind('<Return>', [\&msg_close, $win, $arg{-defbutton}]) 
+    if $arg{-defbutton}; 
 
-    $win->bind( '<Escape>', [ \&msg_close, $win, $arg{-canbutton} ] ) 
-    if $arg{-canbutton} ; 
+    $win->bind('<Escape>', [\&msg_close, $win, $arg{-canbutton}]) 
+    if $arg{-canbutton}; 
 }
 
 #############################
 sub create_text_frame {
-    my $win = shift ;
-    my %arg = @_ ;
+    my $win = shift;
+    my %arg = @_;
 
-    my $Frame ;
+    my $Frame;
 
-    if( $arg{-icon} ) {
-        my $Frame = $win->Frame()->pack( -side => 'left', ) ;
-        &set_icon( $arg{-icon}, $Frame ) if $arg{-icon} ;
+    if ($arg{-icon}) {
+        my $Frame = $win->Frame()->pack(-side => 'left',);
+        &set_icon($arg{-icon}, $Frame) if $arg{-icon};
     }
 
     $Frame = $win->Frame(
-                )->pack( -fill => 'x', -expand => 'y' ) ;
+                )->pack(-fill => 'x', -expand => 'y');
 
-    $Frame->Message( 
-        -text    => $arg{-text},
+    $Frame->Message(
+        -text => $arg{-text},
+        -foreground => $arg{-textfg},
+        -background => $arg{-textbg},
         -justify => $arg{-justify},
-        -aspect  => $arg{-aspect},
-        )->pack( -padx => 5, -pady => 5, -fill => 'x', ) ;
+        -aspect => $arg{-aspect},
+        )->pack(-padx => 5, -pady => 5, -fill => 'x',);
 }
 
 #############################
 sub set_icon {
-    my $icon  = shift ;
-    my $frame = shift ;
+    my $icon  = shift;
+    my $frame = shift;
 
-    my( $text, $colour ) ;
+    my ($text, $colour);
 
-    CASE : {
-        if( $icon eq 'INFO' ) {
-            $text   = 'i' ;
-            $colour = 'black' ;
-            last CASE ;
+    CASE: {
+        if ($icon eq 'INFO') {
+            $text = 'i';
+            $colour = 'black';
+            last CASE;
         }
-        if( $icon eq 'ERROR' ) {
-            $text   = 'E' ;
-            $colour = 'darkred' ;
-            last CASE ;
+        if ($icon eq 'ERROR') {
+            $text = 'E';
+            $colour = 'darkred';
+            last CASE;
         }
-        if( $icon eq 'WARNING' ) {
-            $text   = '!' ;
-            $colour = 'blue' ;
-            last CASE ;
+        if ($icon eq 'WARNING') {
+            $text = '!';
+            $colour = 'blue';
+            last CASE;
         }
-        if( $icon eq 'QUESTION' ) {
-            $text   = '?' ;
-            $colour = 'darkgreen' ;
-            last CASE ;
+        if ($icon eq 'QUESTION') {
+            $text = '?';
+            $colour = 'darkgreen';
+            last CASE;
         }
-        if( $icon eq 'info'    or $icon eq 'error' or 
-            $icon eq 'warning' or $icon eq 'question' ) {
+        if ($icon eq 'info'    or $icon eq 'error' or 
+            $icon eq 'warning' or $icon eq 'question') {
             # Need do nothing.
-            last CASE ;
+            last CASE;
         }
-        DEFAULT : {
-            return ;
+        DEFAULT: {
+            return;
         }
     }
-    if( defined $text ) {
-        $frame->Label( 
+    if (defined $text) {
+        $frame->Label(
             -text => " $text", 
             -font => '-*-times-bold-r-*-*-40-*-*-*-*-*-*-*',
-            -fg   => $colour,
-            )->pack() ;
+            -fg => $colour,
+            )->pack();
     }
     else {
-        $frame->Label( -bitmap => $icon )->pack( -padx => 10, -pady => 10 ) ;
+        $frame->Label(-bitmap => $icon)->pack(-padx => 10, -pady => 10);
     }
 }
 
@@ -215,17 +227,17 @@ sub msg_close {
     # implementation since I've noticed unwanted object references being
     # passed in at other times in other routines - or if its due to me
     # misunderstanding.
-    my $win ;
-    while( ref $_[0] ) {
-        $win = shift ;
-        last if ref $win =~ /MesgBox/o ;
+    my $win;
+    while (ref $_[0]) {
+        $win = shift;
+        last if ref $win =~ /MesgBox/o;
     }
-    my $button = shift ;
+    my $button = shift;
 
-    $win->{-selected_button} = $button ;
+    $win->{-selected_button} = $button;
 }
 
-1 ;
+1;
 
 __END__
 
@@ -235,13 +247,13 @@ MesgBox - Perl/Tk module for message and button dialogue boxes.
 
 =head1 SYNOPSIS
 
-    use MesgBox ; 
+    use MesgBox; 
 
     # Example 1: an 'OK' box.
     # Pressing <Space>, <Return>, <o>, <Control-o> or <Alt-o> will close the box.
 
-    my $msg = $Window->MesgBox( -title => 'Test', -text => 'Test Complete.' ) ;
-    $msg->Show ;
+    my $msg = $Window->MesgBox(-title => 'Test', -text => 'Test Complete.');
+    $msg->Show;
 
     # Example 2: a 'Yes/No' box.
     # Pressing <Space>, <Return>, <y>, <Control-y> or <Alt-y> 
@@ -252,11 +264,11 @@ MesgBox - Perl/Tk module for message and button dialogue boxes.
         -title     => 'Save File?',
         -text      => "Save '$filename'?",
         -icon      => 'QUESTION',
-        -buttons   => [ 'Yes', 'No' ],
+        -buttons   => ['Yes', 'No'],
         -defbutton => 'Yes',
-        ) ;
-    my $ans = $msg->Show ;
-    &save_file if $ans eq 'Yes' ;
+        );
+    my $ans = $msg->Show;
+    &save_file if $ans eq 'Yes';
 
     # Example 3: an 'OK/Cancel' box.
     # Pressing <Space>, <Return>, <o>, <Control-o> or <Alt-o> presses the 'OK'
@@ -267,10 +279,10 @@ MesgBox - Perl/Tk module for message and button dialogue boxes.
     my $msg = $Window->MesgBox(
         -title   => 'Print',
         -text    => 'Print the page',
-        -buttons => [ 'OK', 'Cancel' ],
-        ) ;
-    my $ans = $msg->Show ;
-    &print if $ans eq 'OK' ;
+        -buttons => ['OK', 'Cancel'],
+        );
+    my $ans = $msg->Show;
+    &print if $ans eq 'OK';
 
 =head1 DESCRIPTION
 
@@ -320,6 +332,12 @@ shorter.
 C<-justify> sets the justification for the C<-text> - may be 'center', 'left'
 or 'right'. Defaults to 'center'.
 
+C<-textfg> sets the foreground colour for the C<-text>; default is black.
+
+C<-textbg> sets the background colour for the C<-text>; default is lightgrey.
+
+C<-buttonfont> sets the font for the button; default is 12pt Helvetica.
+
 C<-bindctrl> sets the keyboard bindings to include <Control-$letter>, defaults
 to true.
 
@@ -329,6 +347,19 @@ to true.
 C<-bindord> sets the keyboard bindings to include <$letter>, defaults
 to true.
 
+C<-buttonpadx> sets the left and right padding around the button, defaults to
+0.
+
+C<-buttonpady> sets the above and below padding around the button, defaults to
+0.
+
+C<-buttonfg> sets the button foreground colour, defaults to black.
+
+C<-buttonbg> sets the button foreground colour, defaults to lightgrey.
+
+C<-buttonHL> sets the button highlight colour, defaults to lightgrey.
+
+
 =head1 INSTALLATION
 
 MesgBox.pm should be placed in any Tk directory in any lib directory in
@@ -359,6 +390,9 @@ MesgBox does almost no error checking.
 
 2000/05/05  Minor increment because I accidentally deleted it from CPAN.
 
+2000/09/16  Added new options to control colours and padding: almost all the
+            new code was supplied by Daniel Berger.
+
 =head1 AUTHOR
 
 Mark Summerfield. I can be contacted as <summer@perlpress.com> -
@@ -368,7 +402,7 @@ The code draws very heavily from Stephen O. Lidie's Dialog.pm module.
 
 =head1 COPYRIGHT
 
-Copyright (c) Mark Summerfield 1999. All Rights Reserved.
+Copyright (c) Mark Summerfield 1999,2000. All Rights Reserved.
 
 This module may be used/distributed/modified under the LGPL. 
 
